import numpy as np
import torch
import torch.nn as nn
import os

def ensure_dir(dir_path):
    if not os.path.isdir(dir_path):
        os.mkdir(dir_path)

# ensure_dir(logging_dir)

class Ala2DatasetCV:
    def __init__(self, root_dir="./dataset", size=None, kfold=4):
        """load the whole dataset if size == None"""
        entry = "ala2"
        self.root_dir = root_dir
        self.vac_pdb_file = os.path.join(self.root_dir, entry + ".pdb")
        dataset = np.load(os.path.join(self.root_dir, entry + "_dataset.npz"))
        
        self._dataset_size = len(dataset["coords"])
        if size is None:
            size = self._dataset_size
        else:
            assert self._dataset_size >= size, "desired size exceeds the underlying dataset."
        filepath = root_dir + '/shuffle_indices_%s.npz' % entry
        if not os.path.exists(filepath):
            # make a new shuffling
            indices = np.arange(self._dataset_size)
            np.random.shuffle(indices)
            np.savez(filepath, indices=indices)
        else:
            indices = np.load(filepath)['indices']
            assert len(indices) == self._dataset_size,\
                   'Existing indices does not fit with given dataset.'
            print("Using existing data shuffling.")
        
        self.size = size
        xyzs = dataset["coords"][indices[:size]] # unit: 
        forces = dataset["solvFs"][indices[:size]]
        self._total = torch.utils.data.TensorDataset(torch.Tensor(xyzs),
                                                     torch.Tensor(forces))
        self._kfold = kfold
        self._N_per_fold = len(xyzs) // self._kfold
        
    def get_training_set(self, v_index=0):
        return torch.utils.data.TensorDataset(
            *self._total[np.concatenate(
                [np.arange(i * self._N_per_fold, (i + 1) * self._N_per_fold)
                 for i in range(self._kfold) if i != v_index])])
    
    def get_validation_set(self, v_index=0):
        """Get the `v_index`-th split for validation"""
        return torch.utils.data.TensorDataset(
            *self._total[(v_index * self._N_per_fold):((v_index + 1) * self._N_per_fold)])

    def get_all(self):
        return self._total

# batch loader
class _RandomBatch:
    def __init__(self, data):
        transposed_data = list(zip(*data))
        self.coords = torch.stack(transposed_data[0], 0)
        self.forces = torch.stack(transposed_data[1], 0)

    # custom memory pinning method on custom type
    def pin_memory(self):
        self.coords = self.coords.pin_memory()
        self.forces = self.forces.pin_memory()
        return self

def random_collate(batch):
    # a collate function
    return _RandomBatch(batch)

random_batch_loader = lambda training_set, batch_size: torch.utils.data.DataLoader(training_set, batch_size=batch_size, sampler=torch.utils.data.RandomSampler(training_set), collate_fn=random_collate, pin_memory=True)
batch_loader = lambda training_set, batch_size: torch.utils.data.DataLoader(training_set, batch_size=batch_size, collate_fn=random_collate, pin_memory=True)

